@@ -3,6 +3,7 @@
 #include "edit.h"
 #include <libintl.h>
 #include <iostream>
+#include <libwnck/libwnck.h>
 
 ControlTree *menutree;
 ControlTree *buttontree;
@@ -10,15 +11,21 @@ ControlTree *paneltree;
 ControlTree *edittree;
 ControlTree *widgettree;
 ControlTree *windowtree;
+ControlTree *stoptree;
 ControlTree *dummy; // This one is used to fake another control node
+extern ControlTree *controltree;
 #define _(x) gettext(x)
 
 extern gboolean textentry;
 extern gboolean training;
+extern gboolean quitting;
 extern GtkWidget *the_canvas;
 extern GtkWidget *window;
 
-gboolean panels;
+WnckScreen *wnckscreen;
+
+gboolean panels=FALSE;
+gboolean building=FALSE;
 
 #ifdef GNOME_A11Y
 std::vector<Accessible*> menuitems;
@@ -32,64 +39,53 @@ static AccessibleEventListener* focusListener;
 
 void setupa11y() {
 #ifdef GNOME_A11Y
-  Accessible *tempaccessible;
   focusListener = SPI_createAccessibleEventListener(dasher_focus_listener,
 						    NULL);
-  gboolean success=SPI_registerGlobalEventListener (focusListener, "focus:");
+  SPI_registerGlobalEventListener (focusListener, "focus:");
 #endif
+  // FIXME - should check for correct screen
+  wnckscreen = wnck_screen_get_default();
 }
 
-ControlTree* gettree() {
+void setuptree(ControlTree *tree, ControlTree *parent, ControlTree *children, 
+	       ControlTree *next, void* pointer, int data, std::string string, 
+	       int colour)
+{
+	tree->parent=parent;
+	tree->children=children;
+	tree->pointer=pointer;
+	tree->text=string;
+	tree->next=next;
+	tree->data=data;
+	tree->colour=colour;
+}
+
+ControlTree* gettree() {	
 #ifdef GNOME_A11Y
+  building=TRUE;
   menutree = new ControlTree;
   buttontree = new ControlTree;
   paneltree = new ControlTree;
   edittree = new ControlTree;
   widgettree = new ControlTree;
   windowtree = new ControlTree;
-  Accessible *child;
-  ControlTree *controltree;
   int numchildren;
   desktop = SPI_getDesktop(0);
-  menutree->parent=widgettree;
-  menutree->children=NULL;
-  menutree->pointer=NULL;
-  menutree->data=0;
-  menutree->text=_("Menus");
-  menutree->colour=-1;
-  menutree->next=buttontree;
-  windowtree->parent=NULL;
-  windowtree->children=NULL;
-  windowtree->pointer=NULL;
-  windowtree->data=0;
-  windowtree->text=_("Windows");
-  windowtree->next=paneltree;
-  widgettree->parent=NULL;
-  widgettree->children=menutree;
-  widgettree->pointer=NULL;
-  widgettree->text=_("Application");
-  widgettree->colour=-1;
-  widgettree->next=NULL;
-  buttontree->parent=NULL;
-  buttontree->children=NULL;
-  buttontree->pointer=NULL;
-  buttontree->data=0;
-  buttontree->text=_("Buttons");
-  buttontree->colour=-1;
-  buttontree->next=edittree;
-  paneltree->parent=NULL;
-  paneltree->children=NULL;
-  paneltree->pointer=NULL;
-  paneltree->text=_("Panels");
-  paneltree->colour=-1;
-  paneltree->next=widgettree;
-  edittree->parent=NULL;
-  edittree->children=NULL;
-  edittree->pointer=NULL;
-  edittree->data=0;
-  edittree->text=_("Text");
-  edittree->next=NULL;
-  edittree->colour=-1;
+
+  /* Paneltree
+         |         /-menutree
+     Widgettree------buttontree
+                   \-edittree
+  */
+
+  /* Tree, parent, children, next, pointer, data, string, colour */
+  setuptree(paneltree, NULL, NULL, widgettree, NULL, 0, _("Panels"), -1);
+  setuptree(widgettree, NULL, menutree, NULL, NULL, 0, _("Application"),
+	    -1);
+  setuptree(menutree, widgettree, NULL, buttontree, NULL, 0, _("Menus"), -1);
+  setuptree(buttontree, NULL, NULL, edittree, NULL, 0, _("Buttons"), -1);
+  setuptree(edittree, NULL, NULL, NULL, NULL, 0, _("Text"), -1);
+
   numchildren = Accessible_getChildCount(desktop);
   for (int i=0; i<numchildren; i++) {
     if (findpanels(Accessible_getChildAtIndex(desktop,i))==TRUE) {
@@ -97,6 +93,8 @@ ControlTree* gettree() {
       panels=TRUE;
     }
   }
+
+  widgettree->children = menutree;
   
   if (focusedwindow!=NULL) {
     buildmenutree(focusedwindow,menutree,menus);
@@ -106,11 +104,11 @@ ControlTree* gettree() {
     widgettree->next=buildcontroltree();
   } else {
     widgettree=buildcontroltree();
-    paneltree->next=widgettree;
   }
 #else
   widgettree=buildcontroltree();
 #endif
+  building=FALSE;
   if (panels==TRUE) {
     return paneltree;
   } else {
@@ -120,7 +118,6 @@ ControlTree* gettree() {
 
 #ifdef GNOME_A11Y
 gboolean findpanels(Accessible *parent) {
-  Accessible *child;
   gboolean useful=FALSE;
 
   if (parent==NULL) {
@@ -145,16 +142,18 @@ gboolean findpanels(Accessible *parent) {
 
 ControlTree* buildcontroltree() {
   dummy=new ControlTree;
-  ControlTree *stoptree=new ControlTree;
+  stoptree=new ControlTree;
+  windowtree = new ControlTree;
   ControlTree *pausetree=new ControlTree;
   ControlTree *movetree=new ControlTree;
   ControlTree *deletetree=new ControlTree;
   ControlTree *speaktree=new ControlTree;
-  ControlTree *paneltree=new ControlTree;
+//  ControlTree *paneltree=new ControlTree;
+//  ControlTree *speedtree=new ControlTree;
 #ifndef GNOME_A11Y
   // Otherwise menutree hasn't been set yet, and we end up with a bunch of
   // null pointers rather than children
-  menutree=stoptree;
+  menutree=windowtree;
 #endif
   dummy->pointer=NULL;
   dummy->data=0;
@@ -162,12 +161,21 @@ ControlTree* buildcontroltree() {
   dummy->children=menutree;
   dummy->text=_("Control");
   dummy->colour=8;
+
+  windowtree->pointer=NULL;
+  windowtree->data=0;
+  windowtree->children=buildwindowtree();
+  windowtree->text=_("Windows");
+  windowtree->colour=-1;
+  windowtree->next=stoptree;
+
   stoptree->pointer=(void*)1;
   stoptree->data=2;
   stoptree->children=dummy;
   stoptree->text=_("Stop");
   stoptree->next=pausetree;
   stoptree->colour=242;
+
   pausetree->pointer=(void*)1;
   pausetree->data=3;
   pausetree->children=dummy;
@@ -188,6 +196,7 @@ ControlTree* buildcontroltree() {
     deletetree->text=_("Delete");
     deletetree->colour=-1;
 #ifndef GNOME_SPEECH
+//    deletetree->next=speedtree;
     deletetree->next=NULL;
 #else
     deletetree->next=speaktree;
@@ -197,11 +206,20 @@ ControlTree* buildcontroltree() {
     speaktree->text=_("Speak");
     speaktree->next=NULL;
     speaktree->colour=-1;
+//    speaktree->next=speedtree;
+    speaktree->next=NULL;
 #endif
-  } else {
+/*    speedtree->pointer=(void*)1;
+    speedtree->data=0;
+    speedtree->children=buildspeedtree(speedtree);
+    speedtree->text=_("Speed");
+    speedtree->next=NULL;
+    speedtree->colour=-1;
+*/
+} else {
     pausetree->next=NULL;
-  }
-  return stoptree;
+}
+  return windowtree;
 }
 
 ControlTree* buildmovetree(ControlTree *movetree) {
@@ -261,6 +279,91 @@ ControlTree* buildspeaktree(ControlTree *speaktree) {
   return alltree;
 }
 
+ControlTree* buildspeedtree(ControlTree *speedtree) {
+
+  ControlTree *onetree=new ControlTree;
+  ControlTree *twotree=new ControlTree;
+  ControlTree *threetree=new ControlTree;
+  ControlTree *fourtree=new ControlTree;
+  ControlTree *fivetree=new ControlTree;
+  ControlTree *sixtree=new ControlTree;
+  ControlTree *seventree=new ControlTree;
+  ControlTree *eighttree=new ControlTree;
+  ControlTree *ninetree=new ControlTree;
+  ControlTree *tentree=new ControlTree;
+  
+  onetree->pointer=(void*)1;
+  onetree->data=41;
+  onetree->children=dummy;
+  onetree->text=_("1");
+  onetree->next=twotree;
+  onetree->colour=-1;
+
+  twotree->pointer=(void*)1;
+  twotree->data=42;
+  twotree->children=dummy;
+  twotree->text=_("2");
+  twotree->next=threetree;
+  twotree->colour=-1;
+
+  threetree->pointer=(void*)1;
+  threetree->data=43;
+  threetree->children=dummy;
+  threetree->text=_("3");
+  threetree->next=fourtree;
+  threetree->colour=-1;
+  
+  fourtree->pointer=(void*)1;
+  fourtree->data=44;
+  fourtree->children=dummy;
+  fourtree->text=_("4");
+  fourtree->next=fivetree;
+  fourtree->colour=-1;
+  
+  fivetree->pointer=(void*)1;
+  fivetree->data=45;
+  fivetree->children=dummy;
+  fivetree->text=_("5");
+  fivetree->next=sixtree;
+  fivetree->colour=-1;
+  
+  sixtree->pointer=(void*)1;
+  sixtree->data=46;
+  sixtree->children=dummy;
+  sixtree->text=_("6");
+  sixtree->next=seventree;
+  sixtree->colour=-1;
+  
+  seventree->pointer=(void*)1;
+  seventree->data=47;
+  seventree->children=dummy;
+  seventree->text=_("7");
+  seventree->next=eighttree;
+  seventree->colour=-1;
+  
+  eighttree->pointer=(void*)1;
+  eighttree->data=48;
+  eighttree->children=dummy;
+  eighttree->text=_("8");
+  eighttree->next=ninetree;
+  eighttree->colour=-1;
+  
+  ninetree->pointer=(void*)1;
+  ninetree->data=49;
+  ninetree->children=dummy;
+  ninetree->text=_("9");
+  ninetree->next=tentree;
+  ninetree->colour=-1;
+  
+  tentree->pointer=(void*)1;
+  tentree->data=50;
+  tentree->children=dummy;
+  tentree->text=_("10");
+  tentree->next=NULL;
+  tentree->colour=-1;
+  
+ return onetree;
+}
 ControlTree* builddeletetree(ControlTree *deletetree) {
   ControlTree *forwardtree = new ControlTree;
   ControlTree *backwardtree = new ControlTree;
@@ -332,6 +435,31 @@ ControlTree* builddeletetree(ControlTree *deletetree) {
   return forwardtree;
 }
 
+ControlTree* buildwindowtree()
+{
+	GList *l;
+	wnck_screen_force_update(wnckscreen);
+	ControlTree *firstchild = new ControlTree;
+	ControlTree *tmptree = firstchild;
+	WnckWindow *window;
+	std::string name;
+	GList *windows = wnck_screen_get_windows(wnckscreen);
+
+	for (l = windows; l; l = l->next) {
+		window = (WnckWindow*)l->data;
+		name = wnck_window_get_name (window);
+		setuptree(tmptree, NULL, controltree, NULL, (void *)window, 31, name, -1);
+		tmptree->next = new ControlTree;
+		tmptree->next->parent = tmptree;
+		tmptree = tmptree->next;
+
+	}
+	
+	tmptree->parent->next = NULL;
+	delete tmptree;
+	return firstchild;
+}
+
 #ifdef GNOME_A11Y
 bool buildmenutree(Accessible *parent,ControlTree *ctree,accessibletype Type) {  
   // This code is not desperately nice, and probably ought to be rewritten
@@ -346,7 +474,16 @@ bool buildmenutree(Accessible *parent,ControlTree *ctree,accessibletype Type) {
   // useful ourselves) then return false.
   bool useful=false;
   Accessible *child;
-  ControlTree *childnode;
+
+  AccessibleStateSet *state_set;
+  state_set=Accessible_getStateSet(parent);
+  AccessibleRole role;
+
+  role=Accessible_getRole(parent);
+
+  if (!AccessibleStateSet_contains(state_set,SPI_STATE_ENABLED) && role!=SPI_ROLE_APPLICATION) {
+    return false;
+  }
 
   // This is the node that will represent us if we're useful
   // We don't insert ourselves into the tree just yet, though
@@ -370,30 +507,30 @@ bool buildmenutree(Accessible *parent,ControlTree *ctree,accessibletype Type) {
   } else {
     // We have no kids - check if we're a menu item
     if (Type==menus) {
-      if (Accessible_getRole(parent)==SPI_ROLE_MENU_ITEM||Accessible_getRole(parent)==SPI_ROLE_CHECK_MENU_ITEM) {      
+      if (role==SPI_ROLE_MENU_ITEM||role==SPI_ROLE_CHECK_MENU_ITEM||role==SPI_ROLE_RADIO_MENU_ITEM) {      
 	NewNode->pointer=parent;
 	NewNode->data=1;
-	NewNode->children=menutree;
+	NewNode->children=stoptree;
 	NewNode->text=Accessible_getName(parent);      
 	useful=true;
       } 
     } else if (Type==pushbuttons) {
-      if (Accessible_getRole(parent)==SPI_ROLE_PUSH_BUTTON||Accessible_getRole(parent)==SPI_ROLE_RADIO_BUTTON||Accessible_getRole(parent)==SPI_ROLE_TOGGLE_BUTTON) {
+      if (role==SPI_ROLE_PUSH_BUTTON||role==SPI_ROLE_RADIO_BUTTON||role==SPI_ROLE_TOGGLE_BUTTON) {
 	NewNode->pointer=parent;
 	NewNode->data=1;
-	NewNode->children=menutree;
+	NewNode->children=stoptree;
 	NewNode->text=Accessible_getName(parent);      
 	useful=true;
       } 
     } else if (Type==textenter) {
-      if (Accessible_getRole(parent)==SPI_ROLE_TEXT) {
+      if (role==SPI_ROLE_TEXT) {
 	AccessibleStateSet *state_set;
 	state_set=Accessible_getStateSet(parent);
 	if (AccessibleStateSet_contains(state_set,SPI_STATE_EDITABLE)) {  
 	  AccessibleRelation **relations;
 	  NewNode->pointer=parent;
 	  NewNode->data=30;
-	  NewNode->children=menutree;
+	  NewNode->children=stoptree;
 	  relations=Accessible_getRelationSet(parent);
 	  if (relations == NULL || *relations == NULL) {
 	    NewNode->text=_("Unknown");
@@ -455,13 +592,24 @@ void deletemenutree() {
 void dasher_focus_listener (const AccessibleEvent *event, void *user_data)
 {
   char *name;
-  if (training==TRUE) {
+  // Don't do this if we're in the middle of doing something else - it'll
+  // just result in badness
+  if (training==TRUE || building==TRUE || quitting==TRUE) {
     return;
   }
   
+  building=TRUE;
+
   Accessible *tempaccessible;
+  Accessible *textaccessible=NULL;
   Accessible *accessible = event->source;
   while (dasher_check_window(Accessible_getRole(accessible))!=TRUE) {
+    if (Accessible_getRole(accessible)==SPI_ROLE_TEXT) {
+      AccessibleStateSet *state_set=Accessible_getStateSet(accessible);
+      if (AccessibleStateSet_contains(state_set,SPI_STATE_EDITABLE)) {
+	textaccessible=accessible;
+      }
+    }
     tempaccessible=Accessible_getParent(accessible);
     if (tempaccessible==NULL) {
       break;
@@ -479,12 +627,14 @@ void dasher_focus_listener (const AccessibleEvent *event, void *user_data)
       }
       focusedwindow=accessible;
       deletemenutree();
-      add_control_tree(gettree());
-      dasher_start();
-      set_textbox(NULL);
+      controltree=gettree();
+      add_control_tree(controltree);
+      dasher_start();      
+      set_textbox(textaccessible);
     }
   }
   SPI_freeString(name);
+  building=FALSE;
 }
 
 //FIXME - ripped straight from gok. The same qualms as they have apply.
